
#import logging.handlers
import traceback,re
import sys, argparse,os
import tarfile
import subprocess
import logging
from fastapi import FastAPI
from typing import Union
import logging
from pydantic import BaseModel
import uvicorn
import datetime
from fastapi.middleware.cors import CORSMiddleware
from fastapi import APIRouter, HTTPException
from jinja2 import Environment, FileSystemLoader
#from utils.restApi import RESTClient
import shutil
import json
import threading
import psycopg2
from utils import utils

logging.basicConfig(
    filename='/home/epm/pylog/fastApiApp.log',  # Specify the log file
    filemode='w',  # Append the log messages to the file
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    level=logging.DEBUG  # Set the logging level to DEBUG
)
sysapp = logging.getLogger('createApp')

connection = psycopg2.connect(
    user="postgres",
    password="Wang_2023",
    host="192.168.124.87",
    port=15432,
    database="cmdb"
)
#connection.autocommit = False
# if use manual commit, the transaction might be aborted
#later qury will be aborted
cursor = connection.cursor()

appDir='/data_disk/APP'
buildDir='/data_disk/UPLOAD/sys'
frpcFile='/home/leon/frp_0.35.0_linux_amd64/frpc.ini'

router = APIRouter()

class Item(BaseModel):
    json_str: str

def check_if_line_exists(file_path, target_line):
    with open(file_path, 'r') as file:
        for line in file:
            if target_line in line:
                return True
    return False

# def untar(tar_file_path, destination_dir):
#     with tarfile.open(tar_file_path, 'r') as tar:
#         tar.extractall(path=destination_dir)


def create_app_handler(item_in):
    try:     
        sysapp.info(f"input analysys create_app: {item_in}")
        json_str = item_in.json_str
        json_obj = json.loads(json_str)
        sysapp.debug(f"create_item: {json_obj}")     
        sys_info = json_obj['sysInfo']  
        cust_name = json_obj['cust_name']
        web_port = json_obj['web_port']
        frpc_web_port = json_obj['frpc_web_port']
                
        sysapp.info('del app')
        app_dir=f"{appDir}/{cust_name}"
        cmd=f"pm2 delete {cust_name}_backend"
        subprocess.run(cmd, shell=True, capture_output=True, text=True)
        cmd=f"pm2 delete {cust_name}_etl_worker"
        subprocess.run(cmd, shell=True, capture_output=True, text=True)

        # sysapp.info(f"removedirs {app_dir}")
        # shutil.rmtree(app_dir, ignore_errors=True)        
        # sysapp.info('makedirs')
        # os.makedirs(app_dir,exist_ok=True)
                   
        #find master build
        pattern = r'cons-build-cons_master.*node20.tar$'
        #search_string='master'
        matching_files = [file for file in os.listdir(buildDir) if os.path.isfile(os.path.join(buildDir, file)) and re.search(pattern, file)] 
        file_times = [{'file': file, 'time': os.path.getctime(os.path.join(buildDir, file))} for file in matching_files]
        sysapp.debug(file_times)
        file_times_sorted = sorted(file_times, key=lambda x: x['time'],reverse=True)
        sysapp.debug(file_times_sorted)

        # in case tar corrupted, try next file
        untarResult = False
        for file in file_times_sorted:
            fn = file['file']
            try:              
                sysapp.info(f"removedirs {app_dir}")
                shutil.rmtree(app_dir, ignore_errors=True)        
                sysapp.info('makedirs')
                os.makedirs(app_dir,exist_ok=True)

                sysapp.info(f'untar {buildDir}/{fn}')
                utils.untar(os.path.join(buildDir, fn),app_dir)
                sysapp.info(f'untar {buildDir}/{fn} success')
                untarResult = True
                break
            except Exception as e:
                sysapp.error(f'untar {buildDir}/${fn} error: {e}')

        if not untarResult:
            raise Exception('no valid build file found')
        
        sysapp.info('copy config template')
        config_temp_dir = '/home/epm/git/auto_deploy/cicd/pythonDev'
        config_temp_file = 'confiure-template-pg.json'
        file_loader = FileSystemLoader(config_temp_dir)
        env = Environment(loader=file_loader)
        template = env.get_template(config_temp_file)
        output = template.render(cust_name=cust_name,cust_name_upper=cust_name.upper(),web_port=web_port,pg_ip=sys_info[10])
        sysapp.debug(output)        
        fo = open(os.path.join(app_dir, 'configure.json'), "w")
        fo.write(output)
        fo.close()

        #master_latest_file = max(matching_files, key=lambda x: os.path.getctime(os.path.join(buildDir, x)))
        #sysapp.debug('search_string %s, matching files: %s, master_latest_file: %s'%(search_string,matching_files,master_latest_file))              
        
        original_name='%s/%s'%(app_dir,'backend.js')
        web_new_name=f"{app_dir}/{cust_name}_backend.js"
        os.rename(original_name, web_new_name)
        original_name='%s/%s'%(app_dir,'etl_worker.js')
        etl_new_name=f"{app_dir}/{cust_name}_etl_worker.js"
        os.rename(original_name, etl_new_name)

        sysapp.info('reload')
        cmd="NODE_OPTIONS=--max-old-space-size=32056 DEPLOY_NAME=KEYFUNC_OFFICE pm2 start %s -i 2 --merge-logs"%web_new_name
        result = subprocess.run(cmd, shell=True, capture_output=True, text=True,cwd=app_dir)
        sysapp.info('backend reload: %s %s',cmd,result.stdout) 
        cmd="NODE_OPTIONS=--max-old-space-size=32056 pm2 start %s"%etl_new_name
        result = subprocess.run(cmd, shell=True, capture_output=True, text=True,cwd=app_dir)
        sysapp.info('etl reload: %s %s',cmd,result.stdout) 

        sysapp.info("update frpc")     
        #file_path = 'path_to_your_file.txt'
        #target_line = 'example line'
        if check_if_line_exists(frpcFile, str(frpc_web_port)):
            sysapp.info("The line exists in the file.")
        else:
            sysapp.info("The line does not exist in the file.")  

            # Open the file in append mode
            with open(frpcFile, 'a') as file:
                # Define the lines to append
                lines_to_append = [
                    '\n',
                    '[%s_of1_nodejstest]\n'%cust_name,
                    'type = tcp\n',
                    'local_ip = 127.0.0.1\n',
                    'local_port = %s\n'%web_port,
                    'remote_port = %s\n'%frpc_web_port,
                ]
                # Append each line to the file
                for line in lines_to_append:
                    file.write(line)
            cmd="sudo systemctl restart frpc"
            result = subprocess.run(cmd, shell=True, capture_output=True, text=True)
            sysapp.info('systemctl restart: %s %s',cmd,result.stdout) 

        sysapp.info(f'update cmdb with {sys_info}')
        #configStr = json.dumps(output)
        # let sql = "insert into epm_pm2_service (env_ywid, sys_name, description, is_open, server_id, sys_port, sys_frpc_port, hana_schema, hana_id, sys_type,  configure_json) values ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11) on conflict (env_ywid, sys_name) do update set(env_ywid, sys_name, description, is_open, server_id, sys_port, sys_frpc_port, hana_schema, hana_id, sys_type,  configure_json) = ($12,$13,$14,$15,$16,$17,$18,$19,$20,$21,$22)"
        sql = f"insert into epm_pm2_service(env_ywid, sys_name, description, is_open, server_id, sys_port, sys_frpc_port, hana_schema, hana_id, sys_type,  configure_json) values('{sys_info[0]}','{sys_info[1]}','{sys_info[2]}',{sys_info[3]},'{sys_info[4]}',{sys_info[5]},'{sys_info[6]}','{sys_info[7]}','{sys_info[8]}','{sys_info[9]}','{output}') on conflict (env_ywid,sys_name) do update set(env_ywid, sys_name, description, is_open, server_id, sys_port, sys_frpc_port, hana_schema, hana_id, sys_type,  configure_json) = ('{sys_info[0]}','{sys_info[1]}','{sys_info[2]}',{sys_info[3]},'{sys_info[4]}',{sys_info[5]},'{sys_info[6]}','{sys_info[7]}','{sys_info[8]}','{sys_info[9]}','{output}')"
        sysapp.info(f"sql={sql}")
        cursor.execute(sql)
        #connection.commit()
        sysapp.info("createApp success")
        # rsp["status"] = "success"
        # rsp["config"] = output
    except Exception as e:
        sysapp.error(e)
        return {"error": str(e)}

@router.post("/create_app")
def create_app(item_in: Item):
    rsp={
            "status": "success",
        }
    thread1 = threading.Thread(target=create_app_handler, args=(item_in,)) 
    thread1.start()
    return rsp
    
